/*
    Fichier chargemail.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme tlcharge les mail en attente
    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres et pour fixer le rpertoire racine du systme
    de messagerie.

    Les mail entrants sont stocks dans le sous rpertoire "entree"
    de ce rpertoire racine et se distinguent par leur numro.

    On peut choisir de tlcharger :
    - tous les mails (option par dfaut)
    - un mail particulier identifi par son numro

    Dans les premier cas les mails tlchargs sont supprims du serveur
    Dans le 2me cas le mail est conserv sur le serveur.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include "buflect.h"
#include "pop.h"


/* constantes pour indiquer si l'on efface le(s) mail(s) tlcharg(s) */
#define EFFACE     1
#define CONSERVE   0


/* prototypes */
long num_dermail ();
void chargemail (int numes, long numail, int effacement);
void sauv_num_dermail (long numail);


/* variable globale au source
   (pour viter des tonnes de passages de paramtres) */

char dirmails [80]; // rpertoire de destination des messages


/* programme principal */

int main (int nbarg, char *varg[])
{
    char ficonf [80];   // fichier de configuration du systme de mail
    FILE *fconf;        // descripteur du fichier de configuration
    int  numes, nbmes;  // numro du mail courant et nombre de mails
    long numail;        // numro du mail dans le fichier tlcharg


    // rcuprer le numro du mail  tlcharg s'il a t spcifi
    if (--nbarg > 1 && atoi (varg [1]) > 0)
    {
        numes = atoi (varg [1]);
        nbarg --;
        varg ++;
    }
    else
        numes = 0;  // on tlchargera tous les mails

    // controle du nombre d'arguments
    if (nbarg == 1)
    {
        // rcuprer le nom du fichier de configuration
        strcpy (ficonf, varg [1]);

        if (ficonf [strlen (ficonf) - 4] != '.')
            strcat (ficonf, ".cfg");

        // ouvrir le fichier de configuration
        fconf = fopen (ficonf, "r");

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcuprer le rpertoire racine de la messagerie
                fgets (dirmails, sizeof (dirmails) - 7, fconf);
                dirmails [strlen (dirmails) - 1] = '\0';

                // si ce rpertoire existe
                if (access (dirmails, 0) == 0)
                {
                    // crer si ncessaire le sous repertoire d'entre
                    strcat (dirmails, "/entree");
                    mkdir (dirmails, 0755);

                    // rcupration du numro du dernier fichier mail
                    numail = num_dermail ();

                    // rcupration du nombre de mails
                    nbmes = nbmails ();

                    // tlchargement mail(s)
                    if (numes)
                        // un seul mail demand que l'on laisse sur le serveur
                        chargemail (numes, ++numail, CONSERVE);
                    else
                    {
                        // tlchargement de tous les mails
                        // qu'on supprimera du serveur
                        for (numes = 1; numes <= nbmes; numes++)
                            chargemail (numes, ++numail, EFFACE);
                    }

                    // pour l'affichage
                    putchar ('\n');

                    // sauvegarde du numro du dernier fichier mail
                    sauv_num_dermail (numail);
                }
                else
                    fprintf (stderr,
                     "rpertoire de messagerie %s inexistant\n", dirmails);

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
        else
            fprintf (stderr, "Fichier %s non trouv\n", ficonf);
    }
    else
        fprintf (stderr, "Syntaxe : %s [numro_mail] fichier_configuration\n",
                                                                    varg [0]);

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}


/* retourne le numro du dernier mail tlcharg */

long num_dermail ()
{
    char ficnmail [sizeof (dirmails) + 7];
    FILE *fnmail;
    long numdermail;

    // accs au fichier contenant le numro du dernier mail
    strcpy (ficnmail, dirmails);
    strcat (ficnmail, "/numail");

    fnmail = fopen (ficnmail, "r");

    if (fnmail)
    {
        // recupration de ce numro
        fscanf (fnmail, "%ld", &numdermail);
        fclose (fnmail);

        // retour du numro lu
        return (numdermail);
    }
    else
        // fichier absent : aucun mail n'a encore t tlcharg
        return (-1L);
}


/* tlcharge le message choisi et l'enlve du serveur */

void chargemail (int numes, long numail, int effacement)
{
    char bufw [80];    // buffer d'envoi d'une requte
    char ficmail [sizeof (dirmails) + 9];
    FILE *fmail;

    // cration du fichier mail
    sprintf (ficmail, "%s/r%07ld.n", dirmails, numail);

    fmail = fopen (ficmail, "w");

    if (fmail)
    {
        // message de suivi du droulement
        printf ("\rTlchargement du mail n %d", numes);
        fflush (stdout);

        // demande de lecture du message et rception de l'acquittement
        sprintf (bufw, "RETR %d", numes);
        env_pop (bufw);
        lire_pop ();

        // lire la premire ligne de l'entte du mail
        lire_pop ();

        // lecture et copie du message jusqu' une ligne limite a un .
        while (buf_lect [0] != '.' || buf_lect [1] != '\0')
        {
            // copie de la ligne lue dans le fichier mail
            fputs (buf_lect, fmail);
            fputc ('\n', fmail);     // ncessaire car supprim de buf_lect

            // et lecture de la suivante
            lire_pop ();
        }

        // fermer le fichier contenant le mail
        fclose (fmail);

        // et supprimer si ncessaire le mail du serveur
        if (effacement)
        {
            sprintf (bufw, "DELE %d", numes);
            env_pop (bufw);
            lire_pop ();
        }
    }
    else
        fprintf (stderr, "Impossible d'crire dans le fichier %s\n", ficmail);
}


/* mmorise le numro du dernier mail tlcharg */

void sauv_num_dermail (long numail)
{
    char ficnmail [sizeof (dirmails) + 7];
    FILE *fnmail;

    // accs en criture du fichier contenant le numro du dernier mail
    strcpy (ficnmail, dirmails);
    strcat (ficnmail, "/numail");

    fnmail = fopen (ficnmail, "w");

    if (fnmail)
    {
        // mise  jour du numro
        fprintf (fnmail, "%ld", numail);
        fclose (fnmail);
    }
}
