/*
    Fichier filtresujet.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence.fr

    Traductions des droits d'utilisation dans les fichiers :
    licence.de , licence.en , licence.es , licence.it
    licence.nl , licence.pt , licence.eo , licence.eo-utf


    Ce programme dtruit les fichiers mail comportant une chaine de
    caractres particulire dans le sujet.

    La liste des chaines de caractres interdites est mmorise dans
    un fichier

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include "buflect.h"
#include "pop.h"


#define nbmax_sujet   500     // nombre max de sujets interdits


/* prototypes */
FILE *charge_sujet (FILE *fconf);
void testsujet (int numes);
int  sujet_interdit (char *sujet);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

char *listesujet [nbmax_sujet]; // mmorise les sujets interdits
int  sz_listesujet;     // nombre de sujets mmorises dans listesujet
int  conserves = 0;     // nombre de mails conservs
int  supprimes = 0;     // nombre de mails supprims



/* programme principal */

int main (int nbarg, char *varg[])
{
    char ficonf [80];   // fichier de configuration du systme de mail
    FILE *fconf;        // descripteur du fichier de configuration
    int  numes, nbmes;  // mumro du mail courant et nombre de mails


    if (--nbarg == 1)
    {
        // rcuprer le nom du fichier de configuration
        strcpy (ficonf, varg [1]);

        if (ficonf [strlen (ficonf) - 4] != '.')
            strcat (ficonf, ".cfg");

        // ouvrir le fichier de configuration
        fconf = fopen (ficonf, "r");

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // charger en mmoire les sujets interdits
                if (charge_sujet (fconf))
                {
                    // rcupration du nombre de mails
                    nbmes = nbmails ();

                    // rcupration de la liste des mails
                    for (numes = 1; numes <= nbmes; numes++)
                        testsujet (numes);

                    // dition d'un rcapitulatif
                    printf ("\n%d messages conservs, %d supprims\n",
                                              conserves, supprimes);
                }

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
        else
            fprintf (stderr, "Fichier %s non trouv\n", ficonf);
    }
    else
        fprintf (stderr, "Syntaxe : %s  fich_configuration\n", varg [0]);

    return (0);
}



/* charge en mmoire le fichier des sujets interdits */

FILE *charge_sujet (FILE *fconf)
{
    char ficsujet [90];  // fichier contenant les sujets refuses
    char sujet [80];     // un sujet de ce fichier
    char *nouvsujet;     // sujet retourne alloue dynamiquement
    FILE *frefus;        // descripteur du fichier des sujets
    int i, j;            // compteurs


    // chercher le nom du fichier contenant les sujets interdits
    fgets (ficsujet, sizeof (ficsujet), fconf);
    strcpy (ficsujet + strlen (ficsujet) - 1, "/refus_sujet");

    // initialisation : pas encore de sujet mmorise
    sz_listesujet = 0;

    // accs au fichier des sujets refuses
    frefus = fopen (ficsujet, "r");

    // si ce fichier existe
    if (frefus)
    {
        // rcuprer et mmoriser les diffrents sujets qu'il contient
        while (fgets (sujet, sizeof (sujet), frefus))
        {
            // supprimer le \n en fin de ligne
            sujet [strlen (sujet) - 1] = '\0';

            // allouer une variable pour mmoriser le sujet lu
            nouvsujet = (char *) malloc (strlen (sujet));

            if (! nouvsujet)
            {
                fputs ("Problme d'allocation mmoire\n", stderr);
                exit (0);
            }

            // recopier de sujet
            strcpy (nouvsujet, sujet);

            // mmoriser le sujet dans la liste ou message d'erreur
            if (sz_listesujet < nbmax_sujet)
                listesujet [sz_listesujet ++] = nouvsujet;
            else
            {
                 fprintf (stderr, "%s\n%s\n",
                   "Trop de sujets  filtrer, le filtrage ne sera que partiel",
                   "Recompiler l'application avec nbmax_sujet plus grand");
                return;
            }
        }

        // termin avec ce fichier
        fclose (frefus);
    }
    else
    {
        // message d'erreur si fichier des sujets interdits manquant
        fprintf (stderr, "Pas de fichier %s aucun filtrage possible\n",
                 ficsujet);
    }

    // code de retour du test d'existence du fichier
    return (frefus);
}



/* lit l'entte d'un message, vrifie si le sujet de l'expditeur
   est autoris, et supprime ce message si ce n'est pas le cas. */

void testsujet (int numes)
{
    char bufw [80];       // buffer d'envoi d'une requte
    char bufSubject [80]; // buffer contenant le sujet de l'expditeur
    char motcle [] = "Subject"; // dbut ligne contenant le sujet
    int i, j;             // simples compteurs


    printf ("\rMessage %d", numes);
    fflush (stdout);

    // initialisation
    bufSubject [0] = '\0';

    // demande de lecture de l'entte du message
    sprintf (bufw, "TOP %d 1", numes);
    env_pop (bufw);

    // lecture entte du message et mmorisation de le sujet de l'expditeur
    do
    {
        // lire une ligne de l'entte du message
        lire_pop ();

        // initialisations
        i = 0;
        j = 0;

        // on cherche si on est sur la ligne commenant par Subject:
        while (motcle [i] && ((buf_lect [i] | 0x20) == (motcle [i] | 0x20)))
            i++;

        // le mot cl  Subject  doit tre suivi de  :  dans la ligne lue
        if (buf_lect [i] == ':')
        {
            // recherche du dbut du sujet
            do
                i++;
            while (buf_lect [i] == ' ');

            // tronquer si ncessaire le sujet (pour viter un plantage)
            if (strlen (buf_lect + i) > sizeof (bufSubject))
                 buf_lect [i + sizeof (bufSubject) - 1] = '\0';

            // mmorisation du sujet
            strcpy (bufSubject, buf_lect + i);
        }
    }
    // lecture termine si ligne limite  un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // destruction des mails contenant un sujet interdit
    if (sujet_interdit (bufSubject))
    {
        sprintf (bufw, "DELE %d", numes);
        env_pop (bufw);
        lire_pop ();
        supprimes ++;
    }
    else
        conserves ++;
}



/* recherche si le sujet de l'expditeur est un sujet interdit */

int sujet_interdit (char *sujet)
{
    char *interdit;  // sujet interdit  tester
    int i, j, k;     // compteurs

    // on compare le sujet expditeur avec toutes les sujets  refuser
    for (i = 0; i < sz_listesujet; i++)
    {
        // permettra d'viter des calculs de dplacements dans tableau
        interdit = listesujet [i];

        // initialisation
        k = 0;

        // vrifier si la chaine interdite se trouve dans le sujet
        do
        {
            j = 0;

            while (sujet [j+k] == interdit [j])
                // termin si tous les caractres de la chaine interdite trouvs
                if (interdit [++j] == 0)
                    return 1;
        }
        // sinon recherche dans la suite du sujet
        while (sujet [k++]);
    }

    // le sujet du mail ne fait pas partie des sujets interdits
    return (0);
}
